function [x_p, S_p, x_f, S_f, x_s, S_s] = kalman(y, A, C, S_v, S_w, x0, S_x0, U, B, D)
% Obycejny Kalmanuv filtr pro stavu linearniho systemu.
% Y ... data = radky
% model je tvaru
% x(t+1) = Ax(t) + Bu(t) + v(t)
% y(t)   = Cx(t) + Du(t) + w(t)

% delka dat
n = length(y(1,:));


n_x=length(A);
n_y=size(C,1);

if nargin==7
    U=zeros(1,n);
    B=zeros(n_x,1);
    D=zeros(n_y,1);
end

x_p=zeros(n_x,n+1);
x_f=zeros(n_x,n);
x_s=zeros(n_x,n);
S_p=zeros(n_x,n_x,n+1);
S_f=zeros(n_x,n_x,n);
S_s=zeros(n_x,n_x,n);
K=zeros(n_x,n_y,n);
F=zeros(n_x,n_x,n-1);

% pocatecni hodnoty mi(0|-1) a sigma(0|-1)
x_p(:,1) = x0;
S_p(:,:,1) = S_x0;

% KF
for t = 1:n
    K(:,:,t)   = S_p(:,:,t) * C' * inv (C * S_p(:,:,t) * C' + S_w);
    x_f(:,t)   = x_p(:,t) + K(:,:,t) * (y(:,t) - C * x_p(:,t) - D*U(:,t));
    S_f(:,:,t) = S_p(:,:,t) - K(:,:,t) * C * S_p(:,:,t);
    x_p(:,t+1)   = A * x_f(:,t) + B*U(:,t);
    S_p(:,:,t+1) = A * S_f(:,:,t) * A' + S_v;
end

% smoothing

x_s(:,n)   = x_f(:,n);
S_s(:,:,n) = S_f(:,:,n);

clear t;
for t = n-1:-1:1
    F(:,:,t)     = S_f(:,:,t) * A' * inv( S_p(:,:,t+1) );
    x_s(:,t)   = x_f(:,t) + F(:,:,t) * (x_s(:,t+1) - x_p(:,t+1));
    S_s(:,:,t) = S_f(:,:,t) - F(:,:,t) * (S_p(:,:,t+1) - S_s(:,:,t+1) ) * (F(:,:,t))';
end
